/* Main.cpp - for Screet test app */
/* Copyright 2002 Steve Folta */
/* See the "License" file */

#include "ScreetLexer.h"
#include "ScreetParser.h"
#include "ScreetContext.h"
#include "Screetable.h"
#include "ScreetListener.h"
#include <stdio.h>
#include <stdlib.h>
#include <readline/readline.h>
#include <readline/history.h>


extern const char* tokenNames[];

extern void LexTest(int argc, char* argv[]);
extern void ParseTest();
extern void ListenerTest();


int main(int argc, char* argv[])
{
#if defined(LISTENER_TEST)
	ListenerTest();
#elif defined(PARSE_TEST) || defined(SCREET_PROMPT)
	ParseTest();
#else
	// default
	ParseTest();
#endif

	return 0;
}


class FooHolder : public Screetable {
public:
	FooHolder() : foo("Foo's Initial Value") {}

	ScreetString	screetCall(ScreetString name, int numArgs,
	            	           ScreetString* args);

protected:
	ScreetString	foo;
};

ScreetString FooHolder::screetCall(ScreetString name, int numArgs,
                                   ScreetString* args)
{
	if (name == "foo" && numArgs == 0)
		return foo;
	else if (name == "foo-set" && numArgs == 1) {
		// we'd do "return (foo = args[0]);", but there seems to be a compiler bug...
		foo = args[0];
		return foo;
		}

	else if (name == "action" && numArgs == 0) {
		printf("Bang!\n");
		return ScreetString();
		}

	else {
		char numArgsStr[64];
		sprintf(numArgsStr, "%d", numArgs);
		return ScreetString("##!! FooHolder doesn't know \"") + name + "\" with " +
		       numArgsStr + " arguments. !!##";
		}
}




void ParseTest()
{
	ScreetContext context;
	FooHolder fooHolder;
	context.set("foo-holder", fooHolder.asScreetString());

	while (true) {
		char* line = readline("screet> ");
		if (line == NULL)
			break;
		if (line[0] == 0) {
			free(line);
			break;
			}

		ScreetString text(line);
		ScreetParser parser(text, &context);
		printf("%s\n\n", parser.parse().c_str());

		add_history(line);

		free(line);
		}
}


void ListenerTest()
{
	ScreetContext context;
	FooHolder fooHolder;
	context.set("foo-holder", fooHolder.asScreetString());

	ScreetListener screetListener(&context);

	if (!screetListener.isValid()) {
		perror("Couldn't open the listening socket!");
		return;
		}

	screetListener.listen();
}


void LexTest(int argc, char* argv[])
{
	// get the filename from the args
	if (argc < 2) {
		printf("Usage: %s <file>.\n", argv[0]);
		return;
		}
	const char* fileName = argv[1];

	// read the file
	FILE* file = fopen(fileName, "r");
	if (file == NULL) {
		perror("Couldn't open file");
		return;
		}
	fseek(file, 0, SEEK_END);
	int fileSize = ftell(file);
	rewind(file);
	char* text = new char[fileSize];
	fread(text, 1, fileSize, file);
	fclose(file);

	// lex it
	ScreetLexer lexer(ScreetString(text, fileSize));
	while (true) {
		ScreetToken token = lexer.nextToken();

		switch (token.type) {
			case IDToken:
			case NumberToken:
			case StringLiteralToken:
				printf("  %s\t%s\n", tokenNames[token.type], token.text.c_str());
				break;

			default:
				printf("  %s\n", tokenNames[token.type]);
				break;
			}

		if (token.type == EOFToken)
			break;

		lexer.consumeToken();
		}
}




const char* tokenNames[] = {
	"IDToken",
	"SemicolonToken",
	"EqualsToken",
	"IfToken",
	"ElseToken",
	"WhileToken",
	"DoToken",
	"LoopToken",
	"OpenBraceToken",
	"CloseBraceToken",
	"BreakToken",
	"ContinueToken",
	"ReturnToken",
	"LogicalAndToken",
	"LogicalOrToken",
	"InclusiveOrToken",
	"ExclusiveOrToken",
	"AndToken",
	"EqualityToken",
	"InequalityToken",
	"LessThanToken",
	"GreaterThanToken",
	"LessThanOrEqualToken",
	"GreaterThanOrEqualToken",
	"ShiftLeftToken",
	"ShiftRightToken",
	"PlusToken",
	"MinusToken",
	"MultiplyToken",
	"DivideToken",
	"ModuloToken",
	"NotToken",
	"TwiddleToken",
	"DotToken",
	"OpenParenToken",
	"CloseParenToken",
	"OpenIndexToken",
	"CloseIndexToken",
	"ThisToken",
	"NilToken",
	"TrueToken",
	"FalseToken",
	"NumberToken",
	"StringLiteralToken",
	"CommaToken",
	"EOFToken"
};


