/* ScreetLexer.h - Lexer for Screet */
/* Copyright 2002 Steve Folta */
/* See the "License" file */

#ifndef _ScreetLexer_h_
#define _ScreetLexer_h_

#include "ScreetString.h"


enum ScreetTokenType {
	IDToken,
	SemicolonToken,
	EqualsToken,
	IfToken,
	ElseToken,
	WhileToken,
	DoToken,
	LoopToken,
	OpenBraceToken,
	CloseBraceToken,
	BreakToken,
	ContinueToken,
	ReturnToken,
	LogicalAndToken,
	LogicalOrToken,
	InclusiveOrToken,
	ExclusiveOrToken,
	AndToken,
	EqualityToken,
	InequalityToken,
	LessThanToken,
	GreaterThanToken,
	LessThanOrEqualToken,
	GreaterThanOrEqualToken,
	ShiftLeftToken,
	ShiftRightToken,
	PlusToken,
	MinusToken,
	MultiplyToken,
	DivideToken,
	ModuloToken,
	NotToken,
	TwiddleToken,
	DotToken,
	OpenParenToken,
	CloseParenToken,
	OpenIndexToken,
	CloseIndexToken,
	ThisToken,
	NilToken,
	TrueToken,
	FalseToken,
	NumberToken,
	StringLiteralToken,
	CommaToken,
	EOFToken
};


struct ScreetToken {
	ScreetTokenType	type;
	ScreetString   	text;

	ScreetToken(ScreetTokenType typeIn, const ScreetString& textIn)
		: type(typeIn), text(textIn) {}
	ScreetToken()
		: type(EOFToken) {}
	ScreetToken& operator=(const ScreetToken& other)
		{ type = other.type; text = other.text; return *this; }
};


class ScreetLexer {
public:
	ScreetLexer(ScreetString textIn);
	~ScreetLexer();

	ScreetToken	nextToken();
	ScreetToken	secondToken();
	void       	consumeToken();

protected:
	ScreetString	text;
	ScreetToken 	theNextToken;
	ScreetToken 	theSecondToken;
	const char* 	curCharPtr;

	ScreetToken	getNextToken();
};


#endif
