/* ScreetMethodCall.woosh.cpp - Screet access to Woosh objects */
/* Copyright 2002 Steve Folta */
/* See the "License" file */

#include "ScreetMethodCall.woosh.h"
#include "ScreetMethodCall.h"
#include "ScreetArgumentList.h"
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdlib.h>


ScreetString screetMethodCall(ScreetString object, ScreetString methodName,
                              ScreetArgumentList* arguments)
{
	static const char* callCommandName = "call";

	ScreetString result;

	// create the pipe to read the program's output
	int pipeFD[2];
	pipe(pipeFD);

	// fork
	pid_t pid = fork();
	if (pid == -1) {
		return ScreetString("##!! Couldn't fork! !!##");
		}

	if (pid != 0) {
		// this is still the parent process

		// close the output side of the pipe
		close(pipeFD[1]);

		// wait for the command to finish
		int status;
		pid_t waitResult = waitpid(pid, &status, 0);
		if (waitResult == -1) {
			return ScreetString("##!! Wait failed. !!##");
			}

		// report errors
		if (status != 0) {
			return ScreetString("##!! \"") + methodName + "\" call failed (" +
			       ScreetString(status) + "). !!##";
			}

		// read the output
		static const int bufferSize = 1024;
		char* buffer = new char[bufferSize];
		while (true) {
			ssize_t bytesRead = read(pipeFD[0], buffer, bufferSize);
			if (bytesRead == 0)
				break;
			result += ScreetString(buffer, bytesRead);
			}
		delete buffer;
		}

	else {
		// this is the child process

		// close the input side of the pipe
		close(pipeFD[0]);

		// use the pipe as stdout
		dup2(1, pipeFD[1]);

		// put together the arguments
		int numArgs = arguments->numArguments();
		const char* argv[numArgs + 4];
		argv[0] = callCommandName;
		argv[1] = object.c_str() + 4;		// skip the "##0x"
		argv[2] = methodName.c_str();
		arguments->buildArgv(&argv[3]);
		argv[numArgs + 3] = NULL;

		// run the command
		execvp(callCommandName, (char**) argv);

		// if we got here, it's an error
		exit(1);
		}

	return result;
}


ScreetString screetMethodCall(void* object, ScreetString methodName,
                              int numArgs, ScreetString* args)
{
	/*** !!! Will go away when some stuff is fixed in the parser. !!! ***/
	return "##!! Internal error.  Someday it will be fixed. !!##";
}


