/* ScreetString.h - String object in Screet */
/* Copyright 2002 Steve Folta */
/* See the "License" file */

#ifndef _ScreetString_h_
#define _ScreetString_h_

#include "ScreetCommon.h"


class ScreetStringStorage;


class ScreetString {
public:
	ScreetString()
		: start(NULL), stopper(NULL), storage(NULL) {}
	ScreetString(const char* str);
	ScreetString(const ScreetString& str);
	ScreetString(const char* str, unsigned int numChars)
		: start(str), stopper(str + numChars), storage(NULL) {}
	ScreetString(const char* startIn, const char* stopperIn)
		: start(startIn), stopper(stopperIn), storage(NULL) {}
	ScreetString(long int number, const char* format = NULL);
	~ScreetString();

	ScreetString&	operator=(const ScreetString& other);
	ScreetString&	operator+=(const ScreetString& other);
	ScreetString 	operator+(const ScreetString& other);
	ScreetString	substr(unsigned int firstChar, unsigned int numChars) const;

	bool	operator==(const ScreetString& other) const;
	bool	operator!=(const ScreetString& other) const;
	bool	startsWith(const ScreetString& other) const;
	bool	startsWith(const char* other) const;
	bool	isNumber() const;

	int        	length() const { return stopper - start; }
	bool       	isEmpty() const { return (stopper <= start); }
	const char*	data() const { return start; }
	const char*	end() const { return stopper; }
	const char*	c_str();

	void	setTo(const char* newStart, const char* newStopper);
	void	detach();
	void	semiDetach();	// detaches the string if it has no storage

protected:
	const char*         	start;
	const char*         	stopper;
	ScreetStringStorage*	storage;

	ScreetString(const char* str, unsigned int numChars,
	             ScreetStringStorage* storageIn);
};


#endif
