/* ScreetStringStorage.cpp - Storage for a Screet string */
/* Copyright 2002 Steve Folta */
/* See the "License" file */

#include "ScreetStringStorage.h"
#include "ScreetString.h"
#include <string.h>

/*
	NOTE:
		We automatically add a null at the end -- it's cheap and
		lets us get C-strings easily.
*/

ScreetStringStorage::ScreetStringStorage(const char* start,
                                         unsigned long length)
	: size(length), refCount(1)
{
	text = new char[length + 1];
	memcpy(text, start, length);
	text[length] = 0;
}


ScreetStringStorage::ScreetStringStorage(const ScreetString& str1,
                                         const ScreetString& str2)
	: refCount(1)
{
	unsigned int firstLen = str1.length();
	unsigned int secondLen = str2.length();
	text = new char[firstLen + secondLen + 1];
	memcpy(text, str1.data(), firstLen);
	memcpy(text + firstLen, str2.data(), secondLen);
	text[firstLen + secondLen] = 0;
	size = firstLen + secondLen;
}


ScreetStringStorage::~ScreetStringStorage()
{
	delete text;
}


void ScreetStringStorage::removeReference()
{
	--refCount;
	if (refCount <= 0)
		delete this;
}


