/* ScreetWoosh.cpp - main() for screet-woosh */
/* Copyright 2002 Steve Folta */
/* See the "License" file */

#include "ScreetParser.h"
#include "WooshContext.h"
#include <stdio.h>
#include <stdlib.h>
#include <readline/readline.h>
#include <readline/history.h>


extern void commandLine();
extern void parseScript(const char* fileName, int argc, char** argv);


int main(int argc, char* argv[])
{
	if (argc >= 2) {
		// it's a script
		parseScript(argv[1], argc - 1, &argv[1]);
		}
	else {
		// run a command line
		commandLine();
		}

	return 0;
}


void commandLine()
{
	WooshContext context;

	while (true) {
		char* line = readline("screet> ");
		if (line == NULL)
			break;
		if (line[0] == 0) {
			free(line);
			break;
			}

		ScreetString text(line);
		ScreetParser parser(text, &context);
		printf("%s\n\n", parser.parse().c_str());

		add_history(line);

		free(line);
		}
}


void parseScript(const char* fileName, int argc, char** argv)
{
	// build the context
	WooshContext context;
	context.set("num-args", ScreetString(argc - 1));
	for (int whichArg = 0; whichArg < argc; ++whichArg) {
		ScreetString argName = ScreetString("arg-") + ScreetString(whichArg);
		context.set(argName, argv[whichArg]);
		}

	// open the script file
	FILE* file = fopen(fileName, "r");
	if (file == NULL)
		exit(1);

	// find out the file size
	fseek(file, 0, SEEK_END);
	int fileSize = ftell(file);
	rewind(file);

	// read the file
	char* contents = new char[fileSize];
	int bytesRead = fread(contents, 1, fileSize, file);
	fclose(file);
	if (bytesRead == -1)
		exit(1);

	// parse it
	ScreetString text(contents, fileSize);
	ScreetParser parser(text, &context);
	ScreetString result = parser.parse();
	delete contents;

	// output the result
	fwrite(result.data(), 1, result.length(), stdout);
}


