/* WooshContext.cpp - ScreetContext for Woosh, including objects in the directories */
/* Copyright 2002 Steve Folta */
/* See the "License" file */

#include "WooshContext.h"
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>


ScreetString WooshContext::valueFor(ScreetString name)
{
	// is it an object in the current directory?
	if (isAnObject(name))
		return ScreetString("##0x") + name;

	// is it an object in $O_BASE?
	char* obase = getenv("O_BASE");
	if (obase) {
		if (isAnObject(ScreetString(obase) + "/" + name))
			return ScreetString("##0x") + name;
		}

	return ScreetContext::valueFor(name);
}


bool WooshContext::isAnObject(ScreetString path)
{
	// does the file/directory exist?
	struct stat info;
	int result = stat(path.c_str(), &info);
	if (result != 0)
		return false;

	// is it a directory?
	if (!S_ISDIR(info.st_mode))
		return false;

	// does it have a class?
	result = stat((path + "/._").c_str(), &info);
	if (result != 0 || !S_ISDIR(info.st_mode))
		return false;

	return true;
}


